unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls,
  LConvEncoding, VirtualTrees;

type

  TVirtualTreeColumnsAccess = class(TVirtualTreeColumns);

  TvstRecord = record
    col: TStringArray;
  end;
  PvstRecord = ^TvstRecord;

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Panel1: TPanel;
    vst: TVirtualStringTree;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure vstAdvancedHeaderDraw(Sender: TVTHeader;
      var PaintInfo: THeaderPaintInfo; const Elements: THeaderPaintElements);
    procedure vstDrawText(Sender: TBaseVirtualTree; TargetCanvas: TCanvas;
      Node: PVirtualNode; Column: TColumnIndex; const CellText: String;
      const CellRect: TRect; var DefaultDraw: Boolean);
    procedure vstFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure vstGetNodeDataSize(Sender: TBaseVirtualTree;
      var NodeDataSize: Integer);
    procedure vstGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: String);
    procedure vstHeaderDrawQueryElements(Sender: TVTHeader;
      var PaintInfo: THeaderPaintInfo; var Elements: THeaderPaintElements);
    procedure vstMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  private
    procedure GetDataFromCsv(Datei: String);

  public

  end;

var
  Form1: TForm1;
  HitInfo: THitInfo;   // wird in OnMouseMove gefüllt

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  vst.NodeDataSize:=SizeOf(TvstRecord);
end;


procedure TForm1.vstHeaderDrawQueryElements(Sender: TVTHeader;
  var PaintInfo: THeaderPaintInfo; var Elements: THeaderPaintElements);
begin
  Elements := [hpeBackground];
end;

procedure TForm1.vstAdvancedHeaderDraw(Sender: TVTHeader;
  var PaintInfo: THeaderPaintInfo; const Elements: THeaderPaintElements);
begin
  if hpeBackground in Elements then
  begin
    if Assigned(PaintInfo.Column) then
    begin
      if PaintInfo.Column.Index = HitInfo.HitColumn then
      begin
        PaintInfo.TargetCanvas.Brush.Color := clAqua;
        PaintInfo.TargetCanvas.FillRect(PaintInfo.PaintRectangle);
      end
      else
      begin
        PaintInfo.TargetCanvas.Brush.Color := clActiveBorder;
        PaintInfo.TargetCanvas.FillRect(PaintInfo.PaintRectangle);
      end;
    end;
    if Assigned(PaintInfo.Column) then
    begin
      PaintInfo.TargetCanvas.Brush.Color := clActiveBorder;
      PaintInfo.TargetCanvas.FrameRect(PaintInfo.PaintRectangle);
    end;
  end;
end;

procedure TForm1.vstDrawText(Sender: TBaseVirtualTree; TargetCanvas: TCanvas;
  Node: PVirtualNode; Column: TColumnIndex; const CellText: String;
  const CellRect: TRect; var DefaultDraw: Boolean);
begin
  if Column = 0 then
  begin
    if Node = HitInfo.HitNode then
    begin
      TargetCanvas.Brush.Color := clAqua;
      TargetCanvas.FillRect(CellRect);
    end;
  end;
end;

procedure TForm1.vstMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var  i : Integer;
begin
  vst.GetHitTestInfoAt(x,y,true,HitInfo);
  vst.Refresh;
end;

procedure TForm1.vstFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
var Data : PvstRecord;
begin
  Data := vst.GetNodeData(Node);
  if Assigned(Data) then
  begin
    FillChar(Data^,Sizeof(TvstRecord),' ');
  end;
end;

procedure TForm1.vstGetNodeDataSize(Sender: TBaseVirtualTree;
  var NodeDataSize: Integer);
begin
   vst.NodeDataSize:=SizeOf(TvstRecord);
end;

procedure TForm1.vstGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: String);
var Data : PvstRecord;
begin
  Data := vst.GetNodeData(Node);
  celltext := data^.col[Column];
end;

procedure TForm1.GetDataFromCsv(Datei: String);
var sZeile: String;
    i, iCol : Integer;
    vtc : TVirtualTreeColumn;
    xNode : PVirtualNode;
    Data: PvstRecord;
    tslCSV: TStringList;
    sa : TStringArray;
begin
  tslCSV := TStringList.Create;
  try
    tslCSV.LoadFromFile(Datei);
    vst.Clear;
    vst.Header.Columns.Clear;
    sZeile := ConvertEncoding(tslCSV.Strings[0],GuessEncoding(tslCSV.Strings[0]), EncodingUTF8);
    sa := sZeile.Split(';');
    for i := 0 to HIGH(sa)-1 do vtc := vst.Header.Columns.Add;
    for i := 0 to HIGH(sa)-1 do
    begin
      vst.Header.Columns[i].Style := vsOwnerDraw;
      vst.Header.Columns[i].Options := [coAllowClick,coAllowFocus,coDraggable,coEnabled,coParentBidiMode,coResizable,coSmartResize,coShowDropMark,coVisible];
      vst.Header.Columns[i].Width:=100;
      vst.Header.Columns[i].Text := sa[i];
    end;
    vst.Header.Options := [hoAutoResize,hoColumnResize,hoDblClickResize,hoDrag,hoOwnerDraw,hoVisible];
    vst.Header.Background:=clActiveBorder;
  // füllen des Treeview aus der TStringlist
    for i := 1 to tslCSV.Count-1 do
    begin
      sZeile := ConvertEncoding(tslCSV.Strings[i],GuessEncoding(tslCSV.Strings[i]), EncodingUTF8);
      sa := sZeile.Split(';');
      XNode  := vst.AddChild(nil);
      Data := vst.GetNodeData(XNode);
//      vst.ValidateNode(XNode, false);
      SetLength(Data^.col, HIGH(sa));
      for iCol := 0 to HIGH(sa)-1 do
          Data^.Col[iCol] := sa[iCol];
    end;
  finally
    tslCSV.Free;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  GetDataFromCsv('Mappe1.csv');
end;



end.

